
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                      */
void  mh_restart (int nummrks,
                  int nped,
                  double **prob,
                  int ***y,
                  int n_compnts,
                  int ng)
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                      */

     /* Written by: AWG                                                       */
     /* Date:       23 April, 2001                                            */
     /* Based on:   Sequential Imputation & M-H restarts                      */
     /* Description:                                                          */
     /*             To improve mixing for the meiosis indicators, a proposal  */
     /*             based on seq. imputation is being used. When such a       */
     /*             proposal is used, the M-H ratio becomes                   */

     /*                          W_T(S^*)                                     */
     /*                      ---------------                                  */
     /*                       W_T(S^{(i)})                                    */

     /*      Such a proposal encourages better mixing of the S's and is       */
     /*             termed a "restart".                                       */

     /*     Note - this has been written with the trait locus linked to the   */
     /*            markers.                                                   */

{
   int   c, sx, ii, jj, kk, err;
   int   rnordr;
   int   cuml_tposit;                   /* cumulative test position       */

   double *seqimp_w_old;                /* weights for S_{.j}^{(i)} :
                                           j = 0,..., nummrks             */
   double *seqimp_w_new;                /* weights for S_{.j}^*
                                           where j = 0,..., nummrks       */
   double **locusrec_cpy;               /* copy of the global locusrec    */
   double w_T_old, w_T_new;             /* Seq. imp weights
                                           for S^i and S^*                */
   double m_h_prob;                     /* Metropolis-Hastings ratio      */
   double **new_prob;
   double rnum;
   double cmlprb;

   /* +-----------------------------------+                                   */
   /* |       VARIABLE ALLOCATION         |                                   */
   /* |       AND INITIALIZATION          |                                   */
   /* +-----------------------------------+                                   */

   /* Allocation of seq. imp. weight arrays                                   */
   seqimp_w_old = calloc (sizeof (double), nummrks + 1);
   alloc_chk (WHERE, seqimp_w_old);
   seqimp_w_new = calloc (sizeof (double), nummrks + 1);
   alloc_chk (WHERE, seqimp_w_new);

   /* Allocation of copy of locusrec                                          */
   locusrec_cpy = calloc (sizeof (double), 2);
   alloc_chk (WHERE, locusrec_cpy);
   for (sx = FEM; sx <= MAL; sx++)
   {
      locusrec_cpy[sx] = calloc (nummrks + 1, sizeof (double));
      alloc_chk (WHERE, locusrec_cpy[sx]);
   }

   /* Set locusrec_cpy to locusrec                                            */
   for (sx = FEM; sx <= MAL; sx++)
      for (jj = 0; jj <= nummrks; jj++)
         locusrec_cpy[sx][jj] = xecomo[sx][0][jj];

   /* Allocatin of new_prob                                                   */
   new_prob = malloc (sizeof (double) * n_compnts + 1);
   alloc_chk (WHERE, new_prob);
   for (ii = 0; ii < n_compnts; ii++)
   {
      new_prob[ii] = calloc (sizeof (double), nummrks + 1);
      alloc_chk (WHERE, new_prob[ii]);
   }

   /* +---------------------+                                                 */
   /* |   SELECT LOCUS      |                                                 */
   /* +---------------------+                                                 */

   rnum = rans ();
   cmlprb = 0.0;
   for (ii = 1; ii <= nummrks; ii++)
   {
      if ((rnum >= cmlprb) && (rnum < cmlprb + mrk_distr[ii]))
         rnordr = ii;
      cmlprb += mrk_distr[ii];
   }

   /* +---------------------------+                                           */
   /* |    CALCULATE W_T FOR      |                                           */
   /* |        S^{(i)}            |                                           */
   /* +---------------------------+                                           */

   /* SET all rec rates in locusrec to unlinked                               */
   for (sx = FEM; sx <= MAL; sx++)
      for (jj = 0; jj <= nummrks; jj++)
         xecomo[sx][0][jj] = 0.5;

   /* SECIFY map for L sampler                                                */
   set_positions (nummrks);

   /* CALCULATE seq. imp. weight AT rnordr                                    */
   set_transmission (loci, chromo[0][rnordr], nummrks + 1);
   seqimp_w_old[chromo[0][rnordr]] =
      locus_mjrpeel (0, 1.0, 1.0, 0, loci + chromo[0][rnordr]);  /* peel only */
   if (seqimp_w_old[chromo[0][rnordr]] == -DBL_MAX)
   {
      fprintf (stderr, "Zero probability in peeling operation\n");
      exit (-1);
   }

   /* CALCULATE seq. imp. weights BACKWARDS from rnordr                       */
   for (kk = rnordr - 1; kk >= 0; kk--)
   {
      for (sx = FEM; sx <= MAL; sx++)
         xecomo[sx][0][kk] = locusrec_cpy[sx][kk];
      set_positions (nummrks);
      set_transmission (loci, chromo[0][kk], nummrks + 1);
      seqimp_w_old[chromo[0][kk]] =
         locus_mjrpeel (0, 1.0, 1.0, 0, loci + chromo[0][kk]);
      if (seqimp_w_old[chromo[0][kk]] == -DBL_MAX)
      {
         fprintf (stderr, "Zero probability in peeling operation\n");
         exit (-1);
      }
   }

   /* CALCULATE seq. imp. weights FORWARDS from rnordr                        */
   for (kk = rnordr + 1; kk <= nummrks; kk++)
   {
      for (sx = FEM; sx <= MAL; sx++)
         xecomo[sx][0][kk - 1] = locusrec_cpy[sx][kk - 1];
      set_positions (nummrks);
      set_transmission (loci, chromo[0][kk], nummrks + 1);
      seqimp_w_old[chromo[0][kk]] =
         locus_mjrpeel (0, 1.0, 1.0, 0, loci + chromo[0][kk]);
      if (seqimp_w_old[chromo[0][kk]] == -DBL_MAX)
      {
         fprintf (stderr, "Zero probability in peeling operation\n");
         exit (-1);
      }
   }

   /* CALCULATE W_T_old                                                       */

   w_T_old = 0.0;
   for (kk = 0; kk <= nummrks; kk++)
      w_T_old += seqimp_w_old[kk];

   /* +--------------------------------------+                                */
   /* |      GENERATE S^* and CALCULATE      |                                */
   /* |        W_T FOR  S^*                  |                                */
   /* +--------------------------------------+                                */

   /* Set all rec rates in locusrec to unlinked                               */
   for (sx = FEM; sx <= MAL; sx++)
      for (jj = 0; jj <= nummrks; jj++)
         xecomo[sx][0][jj] = 0.5;

   /*   Secify map for L sampler                                              */
   set_positions (nummrks);

   /*  CALCULATE seq. imp. weight AT rnordr                                   */
   set_transmission (0, 0, 0);
   seqimp_w_new[chromo[0][rnordr]] =
      locus_mjrpeel (0, 1.0, 1.0, 1, loci + chromo[0][rnordr]);
   if (seqimp_w_new[chromo[0][rnordr]] == -DBL_MAX)
   {
      fprintf (stderr, "Zero probability in peeling operation\n");
      exit (-1);
   }

   (void) set_genes (chromo[0][rnordr]);

   for (c = 0; c < n_compnts; ++c)
      new_prob[c][chromo[0][rnordr]] = seg_pen2 (c, y, Qq[chromo[0][rnordr]],
                                                 chromo[0][rnordr], ng, &err);

   /*  CALCULATE seq. imp. weights BACKWARDS from rnordr                      */
   for (kk = rnordr - 1; kk >= 0; kk--)
   {
      for (sx = FEM; sx <= MAL; sx++)
         xecomo[sx][0][kk] = locusrec_cpy[sx][kk];

      set_positions (nummrks);
      seqimp_w_new[chromo[0][kk]] =
         do_lsmpl (nummrks, chromo[0][kk], new_prob, y, ng, 0);
   }

   /*   CALCULATE seq. imp. weights FORWARDS from rnordr                      */
   for (kk = rnordr + 1; kk <= nummrks; kk++)
   {
      for (sx = FEM; sx <= MAL; sx++)
         xecomo[sx][0][kk - 1] = locusrec_cpy[sx][kk - 1];

      set_positions (nummrks);
      seqimp_w_new[chromo[0][kk]] =
         do_lsmpl (nummrks, chromo[0][kk], new_prob, y, ng, 0);
   }

   /*   CALCULATE W_T_new                                                     */
   w_T_new = 0.0;
   for (kk = 0; kk <= nummrks; kk++)
      w_T_new += seqimp_w_new[kk];

   /* +----------------------------------+                                    */
   /* |      CALCULATE  M-H Ratio        |                                    */
   /* +----------------------------------+                                    */

   m_h_prob = w_T_new - w_T_old;        /* i.e. w_T... are logged             */
   m_h_prob = exp (m_h_prob);           /* removing the logs                  */

   /* +--------------------------------------+                                */
   /* |      ACCEPT/REJECT PRPOSED S         |                                */
   /* +--------------------------------------+                                */

   if (rans () <= m_h_prob)
      /* ACCEPT candiate move                                                 */
   {
      /* Replace xx0 and xx1 in previous_realiz                               */

      for (ii = 0; ii <= nummrks; ii++)
         for (jj = 0; jj <= nped; jj++)
         {
            previous_realiz.xx0[ii][jj] = xx0[ii][jj];
            previous_realiz.xx1[ii][jj] = xx1[ii][jj];
         }

      /* Calculate log P(Y_T | S_intvl, S_{intvl+1}, \lamba_T, intvl)         */
      set_positions (nummrks);
      set_transmission (loci, 0, nummrks + 1);
      previous_realiz.prb_Yt_condtl =
         locus_mjrpeel (0, 1.0, 1.0, 0, loci);  /* peel only                  */
      if (previous_realiz.prb_Yt_condtl == -DBL_MAX)
      {
         fprintf (stderr, "Zero probability in peeling operation\n");
         exit (-1);
      }

      for (ii = 0; ii <= nummrks; ii++)
         for (c = 0; c < n_compnts; c++)
            prob[c][ii] = new_prob[c][ii];
   }

   else
      /* REJECT candidate move                                                */
   {
      /* reset xx0 and xx1 to previous.realiz.xx0 and previous.realiz.xx1     */
      assign_xx (nummrks, nped);

      /* based on reset xx0 and xx1  reset gg0 and gg1 to previous values     */
      for (ii = 0; ii <= nummrks; ii++)
         (void) set_genes (ii);
   }

   /* Adjust tposit frequency                                                 */
   /* This can be outside the MH acceptance loop since the test position      */
   /* is not  position.                                                       */

   /* Determine the cumulative position                                       */
   cuml_tposit = 0;
   for (ii = 0; ii < previous_realiz.intvl; ii++)
      cuml_tposit += NTPosInt[ii];
   cuml_tposit += previous_realiz.tposit;

   tposit_freq[cuml_tposit]++;          /* increment frequency                */

   /* Free memory                                                             */
   free ((void *) seqimp_w_old);
   free ((void *) seqimp_w_new);

   for (sx = FEM; sx <= MAL; sx++)
      if (locusrec_cpy[sx] != 0)
         free (locusrec_cpy[sx]);
   if (locusrec_cpy)
      free (locusrec_cpy);

   for (ii = 0; ii < n_compnts; ii++)
      free ((void *) new_prob[ii]);
   free ((void **) new_prob);

   return;

}

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                        */
void  set_xxgg (int nped,
                int lindx)
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                        */

     /* lindx  - locus index where 0 => trait locus and 1 ... L  
        correspond to marker loci i                                   

        When ever the L sampler is run, the contents of xx0 and xx1 are lost.  
        However, the original contents are contained in preivous_realiz.xx0 and
        previous_realiz.xx1. This subroutine copies the contents of             
        previous_realiz.xx? to xx0 and xx1                                    */
{
   int   kk;

   for (kk = 0; kk <= nped; kk++)
   {
      xx0[lindx][kk] = previous_realiz.xx0[lindx][kk];
      xx1[lindx][kk] = previous_realiz.xx1[lindx][kk];
   }
   (void) set_genes (lindx);

   return;
}
