
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void  update_unlnkd_2_lnkd (int nummrks,
                            int nped,
                            int ***y,
                            double **prob,
                            int ngenes,
                            int n_compnts,
                            double p_lnkd)  /* prob of proposing a linkd move */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

     /* Metropolis-Hasting update: proposing a move from an unlinked trait    */
     /*                            locus to a linked trait locus              */

     /* 1. propose a candidate marker interval, test position and cumulative  */
     /*    test position based upon the normalized weights                    */
     /*    P(Y_T | S_{T-1}, S_{T+1})                                          */
     /* NOTE - this was the old way when pseudo-priors were not being used.   */
     /*    However, with the use of psued-priors, this in effect gives        */
     /*    each test position and equal weighting. This simplifies things     */
     /*    greatly but comes at a computational cost.                         */

     /* 2. generate S_T via the L sampler:                                    */
     /*    i.e. P(Y_T | S_i, S_{i+1}, \lamba_T, \alpha)                       */

     /* 3. calculate the M-H prob:                                            */
     /*    alpha ((S_T, \lamba, \alpha), (S_T*, \lamba*, \alpha*))            */
     /*           = p_unlinked * P(Y_T | S_i, S_j, \lamba*, \alpha*)          */
     /*                        * Prior(\lamba*, \alpha*))                     */
     /*  -------------------------------------------------------------------- */
     /* 1/(Num linked test positions) * P(Y_T | \lamba                        */
     /*             = 0.5, \alpha = 0) * Prior(0.5,0)                         */

     /* 4. randomly select move based on above probability.                   */

{
   int   c, ii;
   int   sx;
   int   ctposit;                       /* candidate marker position
                                           within interval                */
   int   cintvl;                        /* candidate marker interval      */
   int   chrm_ctposit;                  /* candidate marker position
                                           along the chrom.
                                           Required by Morgan             */
   int   cmlposit;                      /* cumulative test position       */

   double rnum;
   double prb_Yt_cndtl;                 /* log Pr(Y_T | S_i, S_i+1,
                                           \lamba, \alpha)                */
   double m_h_prob;                     /* Metropolis-Hastings acceptance
                                           probability                    */
   double **prob_new;

   /* Allocate prob_new                                                       */
   prob_new = malloc (sizeof (double) * n_compnts + 1);
   alloc_chk (WHERE, prob_new);
   for (ii = 0; ii < n_compnts; ii++)
   {
      prob_new[ii] = calloc (sizeof (double), nummrks + 1);
      alloc_chk (WHERE, prob_new[ii]);
   }

   for (c = 0; c < n_compnts; c++)
      for (ii = 0; ii <= nummrks; ii++)
         prob_new[c][ii] = prob[c][ii];

   /*   +-----------------------------------------------------+               */
   /*   |   1. Propose Candidate cumulative test position     |               */
   /*   +-----------------------------------------------------+               */

   /* Randomly select linked test position [1, ..., NTPos]                    */
   chrm_ctposit = intran (1, NTPos);

   /* The following convoluted process derives the  marker interval and       */
   /* test position within the marker interval in which the candidate         */
   /* CUMULATIVE test position resides                                        */

   cmlposit = 0;
   for (ii = 0; ii <= nummrks; ii++)    /* where there are nummrks + 1
                                           possible intervals                 */
   {
      if ((chrm_ctposit > cmlposit)
          && (chrm_ctposit <= cmlposit + NTPosInt[ii]))
      {
         cintvl = ii;
         ctposit = chrm_ctposit - cmlposit;
      }
      cmlposit += NTPosInt[ii];
   }

   /*   +------------------------------------------+                          */
   /*   |   2. Propose candidate S_T               |                          */
   /*   +------------------------------------------+                          */

   for (sx = FEM; sx <= MAL; sx++)
      set_chromosomes (nummrks, Marek[sx], 0, &IntVal[chrm_ctposit],
                       &RecVal[sx][chrm_ctposit], sx);
   set_positions (nummrks);

   prb_Yt_cndtl = do_lsmpl (nummrks, 0, prob_new, y, ngenes, 0);

   /*   +----------------------------------------------+                      */
   /*   |   3. Calculate M-H acceptance prob for move  |                      */
   /*   |         from unlinked to linked              |                      */
   /*   +----------------------------------------------+                      */

   /*      alpha ((S_T, \lamba, \alpha),  (S_T*, \lamba*, \alpha*))           */
   /*            = p_unlinked  *  P(Y_T | S_i, S_j, \lamba*, \alpha*)         */
   /*              * Prior(\lamba*, \alpha*)                                  */
   /* ----------------------------------------------------------------------- */
   /* (1/Num linked test pos)  *  P(Y_T | \lamba = 0.5, \alpha = 0)           */
   /*                          *  Prior(0.5,0)                                */

   m_h_prob = (log (1.0 - p_lnkd) + prb_Yt_cndtl
               + log (PseudoPrior[chrm_ctposit]))
      - (log (1.0 / NTPos) + previous_realiz.prb_Yt_condtl
         + log (PseudoPrior[0]));

   m_h_prob = exp (m_h_prob);

   /*   +-----------------------------------------------+                     */
   /*   |   4. Randomly select move based on M-H prob   |                     */
   /*   +-----------------------------------------------+                     */

   if (m_h_prob > 1)
      m_h_prob = 1;                     /* i.e. min(1, M-H prob)              */
   rnum = rans ();
   if (rnum <= m_h_prob)                /* Accept move                        */
   {
      /* Replace xx0 and xx1 in previous_realiz                               */

      set_previous_realiz (cintvl, ctposit, RecVal[FEM][chrm_ctposit],
                           RecVal[MAL][chrm_ctposit], prb_Yt_cndtl, xx0,
                           xx1, nummrks, nped);

      /* Set prob                                                             */
      for (ii = 0; ii <= nummrks; ii++)
         for (c = 0; c < n_compnts; c++)
            prob[c][ii] = prob_new[c][ii];

      /* Adjust tposit frequency                                              */
      tposit_freq[chrm_ctposit]++;
   }
   else
   {
      /* Else move is rejected and previous values retained                   */

      if (previous_realiz.intvl == 0)
         chrm_ctposit = 0;

      for (sx = FEM; sx <= MAL; sx++)
         set_chromosomes (nummrks, Marek[sx], 0, &IntVal[chrm_ctposit],
                          &RecVal[sx][chrm_ctposit], sx);
      set_positions (nummrks);

      /* reassign xx0 and xx1                                                 */
      assign_xx (nummrks, nped);

      /* reassign gg0 and gg1 based on reassigned xx0 and xx1                 */
      for (ii = 0; ii <= nummrks; ii++)
         (void) set_genes (ii);

      /* Adjust tposit frequency                                              */
      tposit_freq[chrm_ctposit]++;
   }

   /* free memory                                                             */
   for (ii = 0; ii < n_compnts; ii++)
      free ((void **) prob_new[ii]);
   free ((void *) prob_new);

   return;
}
