/*****************************************************************************
  program name: lm_auto.c

     Version to try to adapt to a quantitative trait.

******************************************************************************/

/**
 * @defgroup lm_auto lm_auto
 * @ingroup autozyg
 * @{
 */

#define PNAM  "lm_auto"

#include "autozyg.h"
#include "auto_chk.c"

#define AUTO_CKPNTS 1000

int   main (int argc, char *argv[])
{
   int   i, j, jj, k, ng, s, t, c, h, err;
   int   nloc, ntraits, nmarkers, *loc_flags;
   int ***y;                            /* full genotypic array           */
   double **quant = NULL;
   int **dodo = NULL;
   int   ngenes = 0, nfou;
   int   nsw, sx, *nns;
   int  *perm_sw, *perm_loc;            /* permutation arrays of switches
                                         *   and loci                     */
   int  *gaga;                          /* [g]                            */
   int  *npt;                           /* [g+1]  no. of patterns for
                                         *        gametes                 */
   int **index;                         /* [g+1][cd+1]                    */
   int **label;                         /* [g+1][pt+1]                    */
   int **list;                          /* [g+1][pt+1]                    */
   int   nwpt;                          /* number of binary patterns,
                                         *   and of windows               */
   int  *gw, *wloc;                     /* binary return for locus in
                                         *   window                       */
   double ****wscore;                   /* window scores, each cmpt,
                                         * each scoreset, each window,
                                         *   each pattern
                                         *   assume 1 compnt, 1 scoreset  */
   int **count;                         /* [nc][nl]                       */
   double ****nscore;                   /* [nc][ns][nl][pt]  ibd counts:
                                         *   ea component, ea scoreset,
                                         *   ea locus, ea pattern         */
   int   tot_count = 0;                 /* score by component and locus   */

   double *sss;                         /* [nl]                           */
   double ss;
   int ***recounts;                     /* [nc][nl][6] counts of recombs  */
   double **tot_log_prob_seg;
   double **tot_log_prob_pen;
   double **temp_log_prob_seg;
   double **prob;

   FILE *pedF, *oScorF, *oSeedF;
   FILE *ibdF = NULL;

   int   chkitvl = AUTO_CKPNTS;

   atexit (print_tail);
   PCleanUp = clean_auto;
   atexit (clean_morg);
   print_head (*argv);

   if (!proc_pars (argc, argv, PNAM))   /* process input parameter statements */
      exit (EXIT_FAILURE);

   if (!proc_auto ())                   /* check statements for completeness  */
      exit (EXIT_FAILURE);              /* and consistency
                                         * and prepare controls for lm_auto   */

   if (!proc_seeds ())                  /* set up seed parameters             */
      exit (EXIT_FAILURE);

   if (!proc_fils ())                   /* set up pedigree file parameters    */
      exit (EXIT_FAILURE);

   if (!chk_pedfil ())                  /* checks pedigee file for errors     */
      give_up (ERR_qit);

   if (!mak_opedfil ())                 /* writes new pedigree file,          */
      give_up (ERR_qit);                /* if there are changes               */

   if (!(pedF = open_ped ()))           /* reopens pedigree file              */
      give_up (ERR_qit);

   alloc_ped ();

   input_ped (pedF, 0, dodo, quant);    /*input pedigree and trait data       */

   nloc = NLoci - 1;

   nfou = count_founders ();

   MaxNsw = 2 * (pedsize - nfou);
   nns = malloc ((n_compnts + 1) * sizeof (int));
   alloc_chk (WHERE, nns);

   loc_flags = malloc ((nloc + 1) * sizeof (int));
   alloc_chk (WHERE, loc_flags);

/* set up trait structures */
/* read (1) or set (0) information about the Traits (including markers)   */

   ntraits = set_traits (NamdLoc, nloc, loc_flags, pedF);
   nmarkers = print_traits (ntraits, loc_flags);

   printf (" \n");
   fflush (stdout);

/* now allocate space for genotypes y for our values of pedsize, nloc   */

   y = alloc_y (nmarkers, pedsize);

   if (TraitPheno)
   {
      qdex = calloc (pedsize + 1, sizeof (int));
      alloc_chk (WHERE, qdex);
      trtphen = calloc (pedsize + 1, sizeof (int));
      alloc_chk (WHERE, trtphen);
   }
   /* now set marker data on observed individuals */
   /* marker loci are labelled 1,..., nloc */

#if 0
   if (nmarkers < nloc)
   {
      set_markmap (nloc, nmarkers, loc_flags);
      reset_traitmap (nloc, nmarkers, loc_flags);  /* lm_auto only--
                                                      reset Loc0, Rec0 */
      nloc = nmarkers;
      NLoci = nloc + 1;
   }
#endif
   set_markdat (y, nloc, n_compnts);

   /* set data for the trait */

   set_traitdat (y, qdex, trtphen, pedsize, n_compnts);
   prnt_traitdat (y, pedsize, n_compnts);

   alloc_xxgg (nloc, pedsize);

   /* initialize seeds for rans()       */

   setSeeds (SAMPL);

   /* now set and index switches */

   nsw = list_switches (nloc, nns);
   printf (" Number of meiosis switches to sample is %d\n\n", nsw);
   fflush (stdout);

   /* allocate and set arrays of marker and trait positions 
    * and recombination fractions  */

   alloc_chromo (nloc, 0);

   for (sx = FEM; sx <= MAL; sx++)
      set_chromosomes (nloc, Marek[sx], 0, &Loc0, &Rec0[sx], sx);

   free_marek ();

   /* set up permutation arrays for loci and switches  if Scan (not Step) */

   if (Scan)
   {
      perm_sw = malloc (nsw * sizeof (int));
      alloc_chk (WHERE, perm_sw);
      for (i = 0; i < nsw; ++i)
         perm_sw[i] = i;
      perm_loc = malloc ((nloc + 1) * sizeof (int));
      alloc_chk (WHERE, perm_loc);
      for (i = 0; i <= nloc; ++i)
         perm_loc[i] = i;
   }

   /* set up probands for gene scoring */
   /* if the number of components read is less than the last one for which
      there are proband gametes, reset the number of components to score      */

   NScCmp = set_probgam (n_compnts);

   /* set up recombination patterns and initial probs */

   /* Allocate locus structures    -moved to main program, EAT June 2003 */
   /* if TraitPheno, then by default penetrance is incomplete: EAT 12/2003 */
   allocate_loci (nloc + 1, NAll, Qq, pedF, 1);

   /* Use locus-by-locus sampling or sequential imputation for set_up and 
      for initial xx0, xx1 (maternal and paternal segregation indicators) */

   set_seg_mode (0);

   if (SmplSetup == LSMPL)              /* use locus-by-locus sampling */
      set_lsmpl (0, nloc, y, TraitPheno);

   else                                 /* use sequential imputation (AWG) */
      seq_imp (NSeqImp, pedsize, nloc + 1, y, (double *) 0, 1.0);

   peel_trace = TRACE_LEVEL_0;          /* Turn on trace if level >0 */

   alloc_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                 &tot_log_prob_pen);

   /* set genes from initial switches */
   /* Allocate memory for gen_pen()  for each trait locus  */
   /*  alloc_seg_pen called ONCE ONLY for all markers      */
   /*     alloc of prob array for seg_pen results added here EAT 2/2002 */

   k = 0;
   for (j = 0; j <= nloc; ++j)
   {
      ngenes = set_genes (j);

      if (loci[j].locus_type == TRAIT_LOCUS)
         alloc_gen_pen (ngenes, j);
      else if (!k++)
         alloc_seg_pen (ngenes, nloc + 1, y, &prob);
   }
   for (j = 0; j <= nloc; ++j)
      for (c = 0; c < n_compnts; ++c)
         prob[c][j] = seg_pen2 (c, y, Qq[j], j, ngenes, &err);

   tot_count = 0;

   /* gaga[], index[][], label[][], list[][], npt[] are malloced for the
      number of gametes in the largest set */
   /*  alloc of count and sss added here, EAT 2/2002       */

   /*   alloc_az_prob (&count, &sss);  */
   alloc_ibd (&gaga, &index, &label, &list, &npt, &count, &nscore, &sss);

   /* count total number of patterns ----
      modified EAT, March 2001, now will do only full set of patterns
      for each number of gametes  */

   /* probability now computed separately by component  */

   for (c = 0; c < n_compnts; ++c)
   {
      for (h = 0; h <= nloc; ++h)
      {
         tot_log_prob_seg[c][h] = 0.0;
         tot_log_prob_pen[c][h] = 0.0;
      }
      for (j = 0; j <= nloc; ++j)
      {
         count[c][j] = 0;
         for (jj = 0; jj < 6; ++jj)
            recounts[c][j][jj] = 0;
         for (s = 0; c < NScCmp && s < NScSet[c]; ++s)
            for (k = 0; k <= PTgg[NPrGam[c][s]]; ++k)
               nscore[c][s][j][k] = 0.0;
      }
   }

   tot_count = 0;

   /* count total number of patterns */

   for (j = 1; j <= MxGam; ++j)
      npt[j] = index_ibd (j, 0, 0, index[j], label[j], list[j], ibdF);

   pr_npatsets (npt);

   nwpt = 1;
   for (j = 0; j < NLocWndw; ++j)
      nwpt *= 2;

   if (NLocWndw)
   {
      alloc_wndw (&gw, &wloc, &wscore, NWndw, nwpt);

      for (c = 0; c < NScCmp; ++c)
      {
         for (s = 0; s < NScSet[c]; ++s)
            for (j = 0; j < NWndw; ++j)
               for (k = 0; k < nwpt; ++k)
                  wscore[c][s][j][k] = 0.0;
      }
   }
   else
   {
      wloc = malloc ((NLoci + 1) * sizeof (int));
      alloc_chk (WHERE, wloc);
   }

   alloc_logprob ();

   alloc_msmpl ();

   /* open output scores file, if requested */

   if (FilIt)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   /* try to restart if a rescue file is specified */

   t = 0;
   auto_restart (nscore, tot_log_prob_pen, tot_log_prob_seg, prob,
                 count, recounts, &tot_count, &t);

   prnt_mcmcstrt ();

   /* now do burn-in iterations */

   printf (" Now starting %d iterations of burn-in\n", NBrnIt);
   for (t = 1; t <= NBrnIt; t++)
   {
      if (!(t % ChkIt))
      {
         printf (" Starting burn-in iteration %d\n", t);
         fflush (stdout);
      }
      sampling (t, 0, nloc, nsw, ngenes, perm_loc, perm_sw, y, prob, 0);
   }

   /* now do MCMC sampling, and score results */

   for (t = 0; t < NIt; ++t)
   {
      sampling (t, 0, nloc, nsw, ngenes, perm_loc, perm_sw, y, prob, 0);
      /* MCMC sampling step or scan */

      if (!((t + 1) % ScorIt))
      {
         for (c = 0; c < n_compnts; ++c)
            for (j = nloc; j >= 0; --j)
               ++count[c][j];

         for (c = 0; c < NScCmp; ++c)   /* for ea component & score set       */
            for (s = 0; s < NScSet[c]; ++s)
            {
               for (j = nloc; j >= 0; --j)  /* score ea locus                 */
               {
                  jj = chromo[0][j];
                  wloc[j] = score_genes (jj, NPrGam[c][s], gaga,
                                         index[NPrGam[c][s]],
                                         list[NPrGam[c][s]], ProbIx[c][s],
                                         ProbGam[c][s], nscore[c][s][j],
                                         NPtLab[c][s], PatLab[c][s]);
               }
               if (NLocWndw)
                  score_windows (wscore[c][s], NLocWndw, wloc, nloc);
            }
         /* score tot log-liks & recombs for all components */
         for (c = 0; c < n_compnts; ++c)
         {
            full_seg_prob (nloc, c, recounts, temp_log_prob_seg);
            for (j = 0; j < nloc; ++j)
            {
               tot_log_prob_pen[c][j] += log (prob[c][j]);
               tot_log_prob_seg[c][j] += temp_log_prob_seg[c][j];
            }
            tot_log_prob_pen[c][nloc] += log (prob[c][nloc]);
         }
         ++tot_count;
      }
      if (FilIt && !((t + 1) % FilIt))  /* tests for output to scores fil */
         for (c = 0; c < n_compnts; ++c)
         {                              /* outputs by component           */
            fprintf (oScorF, "%d ", c + 1);
            for (jj = 0; jj <= nloc; ++jj)
            {
               j = revchro[0][jj];
               fprintf (oScorF, "%d ", count[c][j]);  /* (counts, by name)    */
            }
            fprintf (oScorF, "\n");
            for (s = 0; c < NScCmp && s < NScSet[c]; ++s)
            {                           /* then by proband gamete set     */
               ng = NPrGam[c][s];
               for (k = 0; k < npt[ng]; ++k)
               {                        /* then by label and pattern      */
                  fprintf (oScorF, "%d ", s + 1);
                  invert_label (label[ng][k], ng, gaga);
                  fprintf (oScorF, "%d  ", label[ng][k]);
                  for (i = 0; i < ng; ++i)
                     fprintf (oScorF, "%d ", gaga[i]);
                  for (jj = 0; jj <= nloc; ++jj)  /* then by name             */
                  {
                     j = revchro[0][jj];
                     fprintf (oScorF, " %e", nscore[c][s][j][list[ng][k]]);
                  }
                  fprintf (oScorF, "\n");
               }
            }
         }
      if (chkitvl && ((t + 1) % chkitvl == 0))
      {
         auto_dump (nscore, tot_log_prob_pen, tot_log_prob_seg, prob,
                    count, recounts, tot_count, t);
      }
   }
   printf ("\n");

   prnt_mcmcend ();

   for (c = 0; c < n_compnts; ++c)
   {
      pr_cmpnthead (c);

      pr_log_prob_seg (0, nloc, -nns[c + 1], tot_log_prob_seg[c], tot_count);

      pr_log_prob_pen (0, nloc, tot_log_prob_pen[c], tot_count);

      pr_reco_counts (0, nloc, recounts[c]);

      /* for ea set, print probabilities for ea pattern for ea window  */

      for (s = 0; c < NScCmp && s < NScSet[c]; ++s)
      {
         /* format and print headings  */

         pr_probhead ();

         pr_pgamset (c, s);

         pr_lochead (chromo);

         /* format and print data  */

         ng = NPrGam[c][s];

         for (k = 0; k < npt[ng]; ++k)
         {
            ss = 0.0;
            for (j = 0; j < NLoci; ++j)
            {
               sss[j] = nscore[c][s][j][list[ng][k]] / count[c][j];
               ss += sss[j];
            }
            if (ss > 0.0001)
            {
               invert_label (label[ng][k], ng, gaga);
               pr_locprob (ng, gaga, sss, label[ng][k]);
            }
         }
         printf ("\n");

         if (NLocWndw)                  /* **** now output window scores */
         {
            if (NPtLab[c][s])
            {
               pr_prbwhead (c, s);
               pr_wndwpat (c, s);
               pr_wndwhead (NWndw);

               /* format and print data  */

               for (i = 0; i < nwpt; ++i)
               {
                  ss = 0.;
                  for (j = 0; j < NWndw; ++j)
                     if (wscore[c][s][j][i] > ss)
                        ss = wscore[c][s][j][i];
                  if (ss / NIt > 0.0001)
                  {
                     binary (i, NLocWndw, gw);
                     pr_wndwprob (c, s, NWndw, i, gw, wscore);
                  }
               }
               printf ("\n");
            }
         }
      }
   }
   /* ****  now we are done **** */
   fflush (stdout);

   /* Free up allocated memory */
   finish_dumping ();
   free_xxgg ();
   free_y (&y);

   free_ibd (&gaga, &index, &label, &list, &npt, &count, &nscore, &sss);
   if (NLocWndw)
      free_wndw (&gw, &wloc, &wscore, NWndw, nwpt);
   else
      free (wloc);

   free_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                &tot_log_prob_pen);
   locus_peel_free ();
   free_allocate_loci ();
   free_chromo (0);
   free_gen_pen ();
   free_seg_pen (nloc + 1, &prob);
   free_logprob ();
   free_msmpl ();
   free_traits ();

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }
   if (FilIt)
      fclose (oScorF);

   if (nns)
   {
      free (nns);
      nns = NULL;
   }
   if (loc_flags)
   {
      free (loc_flags);
      loc_flags = NULL;
   }
   if (Scan)
   {
      if (perm_sw)
      {
         free (perm_sw);
         perm_sw = NULL;
      }
      if (perm_loc)
      {
         free (perm_loc);
         perm_loc = NULL;
      }
   }
   if (qdex)
   {
      free (qdex);
      qdex = NULL;
   }
   if (trtphen)
   {
      free (trtphen);
      trtphen = NULL;
   }

   exit (EXIT_SUCCESS);
}
