/*   ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

     Header file for lm_bayes
     contains #includes, #defines, typedefs, variables and function prototypes

     +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  */

#ifndef BAYES_H
#define BAYES_H

/**
 * @addtogroup lm_bayes lm_bayes
 * @ingroup autozyg
 * @{
 */

/* #####################  Structure Declarations ####################  */
struct _realization {
  int **xx0;            /* realized female meiosis indicators */
  int **xx1;            /* realized male  meiosis indicators */
  int intvl;            /* marker interval in which the trait locus resides
                           where intvl=0 => unlinked */
  int tposit;           /* position of trait locus within marker interval where
                           1 implies the first test position,
                           2 the second etc. */
  double rec_lft[2];    /* rec rate of trait locus to left flanking marker
                           (0 fem, 1 mal)  (=0.5 if trait locus unlinked) */
  double prb_Yt_condtl; /* P(Y_T | S_intvl, S_{intvl+1}, \lamba_T, intvl) */
};

/* ############### global variable declarations  ################## */

GLOBAL int    *tposit_freq;  /* number of times MCMC sampler has visited a
                                test position */

GLOBAL struct _realization  previous_realiz;  /* contains information relating
                                                 to the previous realization. */
GLOBAL double prb_Yt;        /* log Pr(Y_T) */

GLOBAL double *raoI_est;     /* Rao Blackwellization estimator based on
                                I(\lamba_T == x) */

GLOBAL double *mrk_distr;    /* probability distribution placed on the marker
                                locis according to P(Y_j | S_j) */

/* ###############   Prototypes  ############### */

/* Write subroutines  */
void write_rb (FILE *, double **);

/* Allocates:                   */
void alloc_previous_realiz(int , int );

/* Assigns:                      */
void assign_xx(int, int);
void set_traitdat(int ***, int *, int *, int, int);
void set_xxgg( int nped, int lindx);
void set_previous_realiz(int intvl, int tposit, double fem_rec_lft,
                         double mal_rec_lft, double prb_Yt_condtl,
                         int **xx0, int **xx1, int nmrkloc, int pedsize);

/* Updates:                      */
void update_unlnkd_2_lnkd(int, int, int *** , double **, int, int, double );
void update_loci(double, int, int, int ***, int, double **, int ng );
void update_position(double, int , int , int, int ***, double **, int ngenes,
                     int n_compnts);
void mh_restart(int nummrks, int nped, double **prob, int ***y, int n_compnts,
                int ng);

void linked_proposal(int, int , int , int  *, int *, int *, int *, double *,
                     double *);
void unlinked_proposal(int *, int *, int *);
void calc_raoI(int nummrks, int rao_freq, double *rao_realiz);
void normalize_pseudo_prior(double *pseudo_prior_calc);
void read_pseudo_priors(FILE *, double *);

/* Free allocated memory */
void free_local (int *, int, int);

/* Test functions */
void  incomplete (double *val, const Ind * id, const struct Locus *locus,
                 double lam);



#endif /*BAYES_H*/

/** @} */
