/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*
 *       markers.h:  header file for the Markers/ library                     *
 *                -  included by Markers/ functions and by main programs      *
 *                   calling these functions                                  *
 *::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef MARKERS_H
#define MARKERS_H

#include "incs.h"
#include "nghds.h"

/**
 * @addtogroup markers libmarkers
 * @ingroup Libraries
 * @{
 */

typedef struct _Switch
{
   Ind   ind;
   int   sex;
}
Switch;

typedef struct _Switchlist
{
   struct _Switch *current;
   struct _Switchlist *next;
}
Switchlist;

typedef struct _Switchcmp
{
   int   index;
   struct _Switchlist *meioses;
}
Switchcmp;

/**
 * The maximum number of switches.
 * @code
 *   MaxNsw = 2 * pedsize - nfounders;
 * @endcode
 */
GLOBAL int MaxNsw;

/** The vector of the meioses of the pedigree. */
GLOBAL Switch *meiosis;

/** Linked lists of the meioses, by component. */
GLOBAL Switchcmp *swchlist;

GLOBAL int ***lpflag;         /* [2][nh+1][NMarks]  flags set by set_lpflag
                               * to indicate when log-likelihoods may differ
                               * and must be included in probability ratio
                               * (note: NLoci is NMarks + NTraits)           */

GLOBAL int **xx0, **xx1;      /* arrays of maternal and paternal segregation
                                 indicators at all the loci                  */
GLOBAL int **gg0, **gg1;      /* arrays of all labelled founder genes in
                                 the pedigree                                */

                              /* (note: NLoci is NMarks + NTraits)           */
GLOBAL int ** chromo;         /* [nh+1][NLoci]  indices of loci in ordered
                               * chromosome, for each set of recomb values   */
GLOBAL int ** revchro;        /* [nh+1][NLoci]  positions of loci in ordered
                               * chromosome --  "inverse" of chromo[][]:
                               * (chromo[][pos] = nam, revchro[][nam] = pos) */
GLOBAL double *** xecomo;     /* [2][nh+1][NLoci]  sex-valued recombinations
                               * in ordered chromosome                       */


int   read_gen_dat (FILE *, int **, int **, int *);

void  set_switches (int, int, int *, int *);
void  read_segs (FILE *, int, int, int);
int   list_switches (int, int *);
void  free_switches (void);
int   only_kid (Ind *, Ind *);
void  set_chromosomes (int, double *, int, int *, double *, int);
void  set_lpflag (int, int, int);
void  invert_label (int, int, int *);
int   index_ibd (int, int, int, int *, int *, int *, FILE *);
void  get_genes (int, int, int *, int *, int *);
int   score_ibd (int, int *);
void  print_set_up (int);
void  print_obs_genes (int, int ***);
void  pr_log_prob_seg (int, int, double, double *, double);
void  pr_log_prob_segl(int, int, double, double *, double *, double *, double);
void  pr_log_prob_pen (int, int, double *, int);
void  pr_reco_counts (int, int, int **);
void  prntLodSHead (void);
void  prntLodSPos0 (void);
int   prntLodSPos (int);
int   score_either (int, int, int);
int   score_whichever (int, int, int, int);
void  free_allefrq (void);
void  print_chromosome (int, int, int);
int ***alloc_affhist (void);
void  score_aff (int ***);

int score_genes (int, int, int *, int *, int *, int *, int *, double *,
        int, int *);
void score_windows(double **, int , int *, int );

int   ibd_pattern (int, int *, int *);
int   bin_pattern (int, int, int *);
void  binary (int, int, int *);

void  alloc_chromo (int, int),        free_chromo (int);
void  alloc_lpflag (int, int),        free_lpflag (int);
void  alloc_logprob (void),           free_logprob (void);
void  ali_nc      (int **),           ali_nl      (int **),
      ali_n_nl    (int ***, int),     ali_nc_n    (int ***, int),
      ali_nl_n    (int ***, int),     ali_nc_nl   (int ***),
      ali_nc_nl_n (int ****, int);
void  ald_nl      (double **),        ald_nc_n    (double ***, int),
      ald_nl_n    (double ***, int),  ald_nc_nl   (double ***),
      ald_nc_nl_n (double ****, int), ald_nc_nl_ns(double ****);
void  fre_n       (void **),          fre_n_n     (void ***, int),
      fre_nc_n    (void ***),         fre_nl_n    (void ***),
      fre_nc_nl_n (void ****);

/* following moved from Autozyg (mostly)  EAT  feb 2002 */
void  alloc_ibd   (int **, int ***, int ***, int ***, int **,
                   int ***, double *****, double **);
void  alloc_wndw  (int **, int **, double *****, int, int);
void  alloc_loglik(int ****, double ***, double ***, double ***);
void  alloc_lod   (int **, int ***, double ***,
                   double ***, double ***, double ***, double ***);
void  alloc_lf_rt (int, double ***, double ***, int **, int **);
void  alloc_rb    (int, int,  double **, double ***, double **);

void  free_ibd    (int **, int ***, int ***, int ***, int **,
                   int ***, double *****, double **);
void  free_cst    (void);
void  free_wndw   (int **, int **, double *****, int, int);
void  free_loglik (int ****, double ***, double ***, double ***);
void  free_lod    (int **, int ***, double ***, double ***, double ***,
                   double ***, double ***);
void  set_traitdat(int ***, int *, int *, int, int);
void  prnt_traitdat(int ***, int, int);
int   set_probgam (int);
void  set_pedaff  (int ***, int);
/* print functions for lm_auto and ibddrop */
void  pr_npatsets (int *);
void  pr_pgamset  (int, int);
void  pr_wndwpat  (int, int);
void  pr_cmpnthead(int);
void  pr_probhead (void);
void  pr_prbihead (void);
void  pr_prbwhead (int, int);
void  pr_lochead  (int **);
void  pr_locprob  (int, int *, double *, int);
void  pr_wndwhead (int);
void  pr_wndwprob (int, int, int, int, int *, double ****);

void    alloc_xxgg  (int, int);
void    free_xxgg   (void);
int *** alloc_y     (int, int);
void    free_y      (int ****);
int  ** alloc_genes (int, int);
void    free_genes  (int ***);

/* following routines in set_markdat.c:  EAT March 2002        */

void unset_recval   (double **, double **, double **, double **);
void reset_recval   (int, double **, double **, double **, double **);
void set_markmap    (int , int , int *);
void reset_traitmap (int, int, int*);    /* reset Loc0, Rec0 in lm_auto */
void set_markdat    (int ***, int, int);

/* transferred from Quant: EAT June 2002                     */
double comput_lods (double **, double *, int, int, double *);

double lik_eigen(double **, double *, int, double *);

/** @} */

#endif

