
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*   nghds.h:  neighborhood structures used by polyEM programs         */
/*             also macros and function prototypes for Nghds/ fncs     */
/*             including data interfacing between Pars/ proc_poly()    */
/*             and Nghds/ fncs                                         */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef NGHDS_H
#define NGHDS_H

/**
 * @defgroup nghds libnghds
 * @ingroup Libraries
 * @{
 */

#include "incs.h"
#include "pedchk.h"

typedef struct _Ind {
        struct _Ind *pa,*ma;
        char self[MAXNAMLEN+1];               /* char name */
        int index;
        int component;
        int sex;
        struct _Marlist *marriages;
} Ind;

/*      int ntrips;        removed from _Ind  */
/*      int *trips;        used by PBGibbs    */
/*      int nbips;         (MORGAN V1.1)      */
/*      int *bips;                            */

typedef struct _Component {
        int index;
        struct _Indlist *people;
} Component;

typedef struct _Mar {
        struct _Ind *spouse;
        struct _Indlist *kids;
} Mar;

typedef struct _Marlist {
        struct _Mar *current;
        struct _Marlist *next;
} Marlist;

typedef struct _Indlist {
        struct _Ind *current;
        struct _Indlist *next;
} Indlist;
                                        /* parameters set by Nghds/     */
                                        /*   functions:                 */

GLOBAL int pedsize;                     /* no. of individuals read      */
GLOBAL int namestring;                  /* largest name in pedigree file*/
GLOBAL int n_compnts;                   /* # of components found        */
GLOBAL int max_cpsize;                  /* maximum component size found */

GLOBAL int **data;                      /* was data[MAX_SIZE][7]        */

GLOBAL Ind *nghd;
GLOBAL Component *cmplist;              /* vector of components         */
GLOBAL Ind nullnode;

GLOBAL int *qdex, *idex;                /* pedigree to data indexing;
                                           moved to here, since used by both
                                           PolyEm/Quant and Autozyg/Sample
                                           EAT Jan 2004                  */
                /* will be allocated size (pedsize +1) in input routines */
void alloc_ped(void);
void free_ped(void);
void free_nghds(void);
void free_cmplists(void);
void nghds(int **);
void read_name (FILE *, char *);        /* read in a name               */
void input_ped(FILE *, int, int **, double **);
int neighbour(int, int);
int infect(int);
int components(void);

int get_posit(const char *);
int isNullNode (const Ind *);           /* test if an Ind pointer points to
                                          the nullnode (UNKNOWN individual) */
int count_founders(void);
int intmax(int *aa, int n1, int n2);

double kinship (Ind *, Ind *);
double inbreeding (Ind *);

/* Exact IBD status calculation on two linked loci through recursive
   formula. */
double g4link_idx (int, int, int, int, int, int, int, int,
                   double);
double g4link (Ind *, int, Ind *, int, Ind *, int, Ind *, int, double);

/** @} */

#endif  /*NGHDS_H*/

