/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*           parsepars.h:  header file for the Pars/ library                  */
/*                         -  of file scope                                   */
/*                         -  included by Pars/ functions                     */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef  PARSEPARS_H
#define  PARSEPARS_H

/**
 * @defgroup parse libpars
 * @ingroup Libraries
 * @{
 */

#define SIGSQ(r, c)  SigSq[(r) * (r - 1) /2 + c - 1]
#define TAUSQ(r, c)  TauSq[(r) * (r - 1) /2 + c - 1]
#define SIG_MODL(r, c)  SigModel[(r) * (r - 1) /2 + c - 1]
#define TAU_MODL(r, c)  TauModel[(r) * (r - 1) /2 + c - 1]

static FILTAB_PTR  pfloop, pft;
static FILE * pfile;


/***    limits overriding variables                                         ***/

static int    allw_pedsiz,  allw_cmpsiz,   allw_obsiz;

/***    record variables                                                    ***/

static int parentOrdIn  = UNDEF;
static int parentOrdOut = UNDEF;

/*** the following macro constants are used by proc_pars() and proc_drop()  ***/

#define NCH  0      /* no chromosome          */
#define CH   1      /* chromosome             */
#define NCMP 0      /* no component           */
#define CMP  1      /* component              */
#define NSC  0      /* no scoreset            */
#define SC   1      /* scoreset               */

#define NO_CH_NO -1 /* chrom not needed       */

#define MK  0       /* no. of markers         */
#define AMK 1       /* all markers            */
#define MKL 2       /* marker list            */
#define NTR 0       /* no traits              */
#define TR  1       /* traits                 */

#define RM  0       /* recomb fraction        */
#define DM  1       /* distance               */
#define PM  2       /* position               */
#define IM  3       /* interval proportions   */ 
#define AM  4       /* all interval props     */
#define XRM 5       /* external recomb fracts */
#define XDM 6       /* external distance      */
#define BRM 7       /* begin recomb fracts    */
#define BDM 8       /* begin distances        */
#define ERM 9       /* ending recomb fracts   */
#define EDM 10      /* ending distances       */

#define AV  0       /* additive variance      */
#define RV  1       /* residual variance      */


/***  the following data structures are used by proc_pars() and proc_kin()  ***/
/***  for PedComp/kinship computing requests                                ***/

struct kinship_node {                    /*Kinship_node:                      */
    int component;                       /*for this comp,                     */
    int no_nams;                         /*the no. of probands                */
    char * p_nams;                       /*and a ptr to the list of names     */
    struct kinship_node * p_next;
                    };
typedef struct kinship_node KINS;
typedef KINS * KINS_PTR;
typedef KINS * INBR_PTR;
typedef KINS * INBR2_PTR;
static KINS_PTR  first_kins_node = NULL;
static INBR_PTR  first_inbreed_node = NULL;
static INBR2_PTR first_2loc_node = NULL;


/***  the following data structures are used by proc_pars() and proc_drop() ***/
/***  for genedrop simulation specifications                                ***/
/***    (some are also used by autozyg programs)                            ***/

static int norm_frq_spec = N;           /*Flag - normalize allele freqs       */

struct unlnkd_node {                    /*Unlnkd_node:  unlinked traits       */
    int no;
    struct unlnkd_node * p_next;
                   };
typedef struct unlnkd_node UNLNKD;
typedef UNLNKD * UNLNKD_PTR;
static UNLNKD_PTR first_unlnkd_node = NULL;


struct sim_node {                       /*Sim_node:                           */
    int sim_typ;                        /*simulation/select statement type    */
    int chrom_no;                       /*chromosome no.                      */
    int no_mark_sel;                    /*no. markers selected/simulated      */
                                        /*(for all, added by test_mark_map()) */
                                        /*(1, for only 1 marker)              */
    int no_mark_map;                    /*no. markers mapped (from chrom map) */
                                        /*(added by test_mark_map())          */
                                        /*(0, for only 1 marker)              */
    int no_traits;                      /*no. of linked traits                */
    int * p_mark;                       /*ptr to list of marker no.'s         */
                                        /*(no. is no_mark_sel)                */
    struct sim_node * p_next; 
                 };
typedef struct sim_node SIM;
typedef SIM * SIM_PTR;
static SIM_PTR first_sim_node = NULL;


struct map_node {                       /*Map_node:                           */
    int chrom_no;                       /*chromosome no.                      */
    int no_loci;                        /*no. markers or traits               */
    struct dist_list * p_dist;          /*ptr to list of dists for loci       */
    struct map_node * p_next;           /*3 lists:  female, male and sex-avgd */
                 };
typedef struct map_node MAP;
typedef MAP * MAP_PTR;
static MAP_PTR first_mark_map_node[3] = {NULL, NULL, NULL};
static MAP_PTR first_trait_map_node[3] = {NULL, NULL, NULL};

struct dist_list {                      /*Dist_node:                          */
    int no;                             /*marker or trait no.                 */
    int map_mark;                       /*preceding marker on this chrom      */
    double dist;                        /*dist of marker or trait from the loc*/
    double prob;
                  };
typedef struct dist_list DIST;
typedef DIST * DIST_PTR;


struct vmap_node {                      /*Vmap_node:                          */
    int chrom_no;                       /*for this chromosome,                */
    int no;                             /*the trait no.                       */
    struct xt_map * p_beg;              /*ptr to starting positions           */
    struct xt_map * p_end;              /*ptr to ending positions             */
    struct nt_map_list * p_intv;        /*ptr to list of positns beteewn marks*/
    struct vmap_node * p_next;          /*3 sets:  female, male and sex-avgd  */
                 };
typedef struct vmap_node VMAP;
typedef VMAP * VMAP_PTR;
static VMAP_PTR first_trait_vmap_node[3] = {NULL, NULL, NULL};

struct xt_map {                         /*Xt_map:                             */
    int no_pos;                         /*no. beginning or ending positions   */
    double * p_dist;                    /*ptr to distances (incr order)       */
    double * p_rf;                      /*and recomb fractions                */
                 };                     /*(with first or last marker)         */
typedef struct xt_map XTMAP;
typedef XTMAP * XTMAP_PTR;

struct nt_map_list {                    /*Nt_map_list:                        */
    int intv_no;                        /*interval no. (0 for all),           */
    int no_prop;                        /*no. proportions for this interval,  */
    double * p_prop;                    /*ptr to list of proportions of the   */
                                        /*distance between markers            */
    struct nt_map_list * p_next;        /*ptr to node for next interval       */
                 };
typedef struct nt_map_list NTMAP;
typedef NTMAP * NTMAP_PTR;

struct locus_node {                     /*Locus_node:                         */
    int chrom_no;                       /*chromosome no.                      */
    int no;                             /*the marker or trait no.             */
    int no_alleles;                     /*the no. alleles                     */
    struct freq_node * p_freq_node;     /*ptr to the allele freq node         */
    struct locus_node * p_next;
                    };
typedef struct locus_node LOCUS;
typedef LOCUS * LOCUS_PTR;
static LOCUS_PTR first_mark_locus_node = NULL;
static LOCUS_PTR first_trait_locus_node = NULL;

struct freq_node {                      /*Freq_node:                          */
    int no_loci;                        /*the no. loci with these allele freqs*/
    double sum;                         /*sum of allele frequencies           */
    double * p_freq;                    /*and ptr to list of freqs            */
                 };
typedef struct freq_node FREQ;
typedef FREQ * FREQ_PTR;


struct means_node {                     /*Means_node: the genotype            */
    int no;                             /*the genotype means (3) for the trait*/
    double mean[3];
    struct means_node * p_next;
                   };
typedef struct means_node MEANS;
typedef MEANS * MEANS_PTR;
static MEANS_PTR first_means_node = NULL;


struct vari_node {                      /*Vari_node:                          */
    int no;                             /*trait no.                           */
    double vari;                        /*additive or residual variance       */
    struct vari_node * p_next;
                  };
typedef struct vari_node VARI;
typedef VARI * VARI_PTR;
static VARI_PTR first_addi_node = NULL;
static VARI_PTR first_resi_node = NULL;

static int n_trait_int;                 /*No. of pairs of traits and positns  */

static int * trait_int[2];              /*Trait position list:                */
                                        /*trait no.'s and the corresponding   */
                                        /*integer positions in the ped record */


/***  the following data structures are used by proc_pars() and proc_auto() ***/
/***  for Autozyg/ specifications                                           ***/
/***  A few of these are also used for ibddrop                              ***/

struct markdat_node {                   /*Markdat_node:                       */
    int chrom_no;                       /*for ea chrom                        */
    int no_obs;                         /*no. observed individuals            */
    int no_marks;                       /*no. of markers (1 or no. mapped)    */
    char * p_obs;                       /*ptr to the list of ped member names */
    int * p_alls;                       /*ptr to the observed alleles         */
    struct markdat_node * p_next;
                 };
typedef struct markdat_node MARKDAT;
typedef MARKDAT * MARKDAT_PTR;
static MARKDAT_PTR first_markdat_node = NULL;

struct probgam_node {                   /*Probgam_node:                       */
    int component;                      /*for this component                  */
    int scoreset;                       /*and scoreset,                       */
    int no_pgams;                       /*the no. of proband gametes          */
    struct probgam_list *  p_pglist;    /*and a ptr to the list               */
    struct probgam_node * p_next;
                 };
typedef struct probgam_node PROBGAM;
typedef PROBGAM * PROBGAM_PTR;
static PROBGAM_PTR first_probgam_node = NULL;

struct probgam_list {                    /*Probgam_list:                      */
    char nam[MAXNAMLEN+1];               /*the list of individuals (names)    */
    int gamete;                          /*and gametes                        */
                  };
typedef struct probgam_list PROBANDS;
typedef PROBANDS * PROBLIST_PTR;

struct affnam_node {                    /*Affind_node:                        */
    int component;                      /*for this component                  */
    int no_affs;                        /*the no. of affected individuals     */
    char * p_afflist;                   /*and a ptr to the list of names      */
    struct affnam_node * p_next;
                   };
typedef struct affnam_node AFFNAM;
typedef AFFNAM * AFFNAM_PTR;
static AFFNAM_PTR first_affnam_node = NULL;

struct locwind_node {                   /*Locwind_node:                       */
    int chrom_no;                       /*for this chromosome,                */
    int window;                         /*the no. of loci in ea window        */
    struct locwind_node * p_next;
                  };
typedef struct locwind_node LOCWND;
typedef LOCWND * LOCWND_PTR;
static LOCWND_PTR first_locwind_node = NULL;

struct windpat_node {                   /*Windpat_node:                       */
    int component;                      /*for this component                  */
    int scoreset;                       /*and scoreset,                       */
    int no_pats;                        /*the no. of pattern labels           */
    int *p_pats;                        /*and a ptr to the set                */
    struct windpat_node * p_next;
                     };
typedef struct windpat_node WNDPAT;
typedef WNDPAT * WNDPAT_PTR;
static WNDPAT_PTR first_windpat_node = NULL;

/***     the following variables and tables, used by proc_pars(),           ***/
/***     proc_poly() and check_pars(), contain PolyEM/ modelling            ***/
/***     specifications                                                     ***/

static int    add_cor_flag,             /*Starting values are correlations,   */
              res_cor_flag;             /*not covariances                     */

static int    naddvar, naddcov,         /*No. starting values input           */
              nresvar, nrescov;         /*for variances and covariances       */

static int    nfitadd, nfitres;         /*No. fit flags for covariances       */

static int    seltrff[FMAX_REAL][FMAX_FEF];   /*Selection/nonselection        */
                                        /*of traits & fixed effects.          */
                                        /*Same as ffdex[][], but not          */
                                        /*condensed for traits selected       */

static int    nstartff[FMAX_REAL][FMAX_FEF];   /*No. starting values for      */
                                        /*levels for the fixed effect         */
                                        /*(including mean) for the trait      */

static double startff[FMAX_REAL][FMAX_FEF][LEVELS];    /*Starting values      */
                                        /*for fixed effect levels/means       */

static int    selff[FMAX_FEF];          /*Selection/nonselection of           */
                                        /*fixed effects (incl mean)           */

static int    eigen_set, eigen_compute; /*Eigenvalue options                  */

static int    n_eigen;                  /*No. eigenvalues given               */

static int    n_pseudo;                 /*No. pseudo-priors given             */

/** @} */

#endif  /*PARSEPARS_H*/

