/* functions called by lm_auto, lm_lods, ibddrop for allocating and 
 * freeing arrays  
 *
 *  alloc_y(), alloc_ibd(), alloc_wndw(), alloc_lod(), alloc_loglik()
 *  alloc_lf_rt(), alloc_rb()
 *  free_y(), free_ibd(),  free_wndw(), free_lod(),  free_loglik()
 *                                                              MAJ Aug 01  
 *
 * functions called by lm_lange that are different from Autozyg versions
 *  alloc_loglik(), free_loglik()                               EAT Aug 01  
 *
 *  All above moved to and combined in Msample library          EAT Feb 02
 *     Items from alloc_az_prob and free_az_prob moved to 
 *        alloc/free _seg_pen, _loglik, and _ibd as appropriate        
 *     Part of alloc_ld_prob  moved to alloc_loglik -- remainder
 *        renamed alloc_lod; ditto with the free_*
 * 
 *  allocz_y(), renamed alloc_y();  args changed for alloc_y() and free_y()
 *  alloc_wndw(), free_wndw() added for locus window option for ibddrop
 *                                                              MAJ Apr 02
 *  alloc_wndw(), free_wndw() modified by addition of wloc binary vector
 *                                                              EAT Oct 02
 */

#include "markers.h"

int ***alloc_y (int nl, int nped)
{

   int ***y;

   y = malloc (2 * sizeof (int **));
   alloc_chk (WHERE, y);
   y[0] = alloc_genes (nl, nped);
   y[1] = alloc_genes (nl, nped);

   return y;
}

void  alloc_ibd (int **g, int ***ix, int ***lab, int ***lst, int **np,
                 int ***ct, double *****nsc, double **sss)
{
/*  int g   [G]
 *  int ix  [G+1][CD+1]
 *  int lab [G+1][PT+1]
 *  int lst [G+1][PT+1]
 *  int npt [G+1]
 *  int ct      [nc][nl]            moved here EAT 2/2002
 *  double nsc [NC][NS][NL][PT+1]
 *  double sss   [nl]               moved here EAT 2/2002
 *
 *  see table in limdefs.h for no. gametes/codes/patterns
 */

   int   i, j, c, s;

   *np = malloc ((MxGam + 1) * sizeof (int));
   alloc_chk (WHERE, *np);
   (*np)[0] = 0;

   *g = malloc (MxGam * sizeof (int));
   alloc_chk (WHERE, *g);

   *lab = malloc ((MxGam + 1) * sizeof (int *));
   alloc_chk (WHERE, *lab);
   (*lab)[0] = NULL;

   *ix = malloc ((MxGam + 1) * sizeof (int *));
   alloc_chk (WHERE, *ix);
   (*ix)[0] = NULL;

   *lst = malloc ((MxGam + 1) * sizeof (int *));
   alloc_chk (WHERE, *lst);
   (*lst)[0] = NULL;

   /* the index, list and label arrays are allocated as ragged arrays */

   for (i = 1; i <= MxGam; ++i)
   {
      (*ix)[i] = malloc ((CDgg[i] + 1) * sizeof (int));
      alloc_chk (WHERE, (*ix)[i]);
      (*lst)[i] = malloc ((PTgg[i] + 1) * sizeof (int));
      alloc_chk (WHERE, (*lst)[i]);
      (*lab)[i] = malloc ((PTgg[i] + 1) * sizeof (int));
      alloc_chk (WHERE, (*lab)[i]);
   }
   *nsc = malloc (NScCmp * sizeof (double ***));
   alloc_chk (WHERE, *nsc);

   /* the nscore array is ragged in 2 dimensions */

   for (c = 0; c < NScCmp; ++c)
      if (c < NScCmp && NScSet[c])
      {
         (*nsc)[c] = malloc (NScSet[c] * sizeof (double **));
         alloc_chk (WHERE, (*nsc)[c]);
         for (s = 0; s < NScSet[c]; ++s)
         {
            (*nsc)[c][s] = malloc (NLoci * sizeof (double *));
            alloc_chk (WHERE, (*nsc)[c][s]);
            for (j = 0; j < NLoci; ++j)
            {
               (*nsc)[c][s][j] = malloc ((PTgg[NPrGam[c][s]] + 1) *
                                         sizeof (double));
               alloc_chk (WHERE, (*nsc)[c][s][j]);
            }
         }
      }
      else
         (*nsc)[c] = NULL;

   ali_nc_nl (ct);
   ald_nl (sss);
   return;
}

void  alloc_wndw (int **g, int **wl, double *****nsc, int nw, int nwpt)
{
/* int  g[W]
 * double nscore[NC][NS][NW][NWPT]
 */
   int   c, s, k;

   *g = malloc (NLocWndw * sizeof (int));
   alloc_chk (WHERE, *g);

   *wl = malloc ((NLoci + 1) * sizeof (int));
   alloc_chk (WHERE, *wl);

   /* the nscore array is ragged in 2 dimensions */

   *nsc = malloc (NScCmp * sizeof (double ***));
   alloc_chk (WHERE, *nsc);

   for (c = 0; c < NScCmp; ++c)
      if (NScSet[c])
      {
         (*nsc)[c] = malloc (NScSet[c] * sizeof (double **));
         alloc_chk (WHERE, (*nsc)[c]);
         for (s = 0; s < NScSet[c]; ++s)
         {
            (*nsc)[c][s] = malloc (nw * sizeof (double *));
            alloc_chk (WHERE, (*nsc)[c][s]);
            for (k = 0; k < nw; ++k)
            {
               (*nsc)[c][s][k] = malloc (nwpt * sizeof (double));
               alloc_chk (WHERE, (*nsc)[c][s][k]);
            }
         }
      }
      else
         (*nsc)[c] = NULL;
   return;
}

void  alloc_lf_rt (int npos, double ***r_l, double ***r_r,
                   int **l_l, int **l_r)
{
/* double rec_left[2][NLocPos + 1], double rec_right[2][NLocPos + 1],
   int    loc_left[NLocPos + 1],    int    loc_right[NLocPos + 1]             */

   int   i;

   *r_l = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, *r_l);
   *r_r = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, *r_r);
   for (i = 0; i < 2; ++i)
   {
      (*r_l)[i] = malloc (npos * sizeof (double));
      alloc_chk (WHERE, (*r_l)[i]);
      (*r_r)[i] = malloc (npos * sizeof (double));
      alloc_chk (WHERE, (*r_r)[i]);
   }
   *l_l = malloc (npos * sizeof (int));
   alloc_chk (WHERE, *l_l);
   *l_r = malloc (npos * sizeof (int));
   alloc_chk (WHERE, *l_r);

   return;
}

void  alloc_rb (int npos, int nb, double **rb, double ***rbb, double **rbs)
{
/* double rb_prob[NTPos + NMarks + 1], 
   double rb_prbm[20][NTPos + NMarks + 1],
   double rb_sd[NTPos + NMarks + 1]                                           */

   int   i;

   *rb = malloc (npos * sizeof (double));
   alloc_chk (WHERE, *rb);

   *rbb = malloc (20 * sizeof (double *));
   alloc_chk (WHERE, *rbb);
   for (i = 0; i < 20; ++i)
   {
      (*rbb)[i] = malloc (npos * sizeof (double));
      alloc_chk (WHERE, (*rbb)[i]);
   }

   *rbs = malloc (npos * sizeof (double));
   alloc_chk (WHERE, *rbs);

   return;
}

void  alloc_lod (int **sc, int ***lsf, double ***lrat, double ***clik,
                 double ***pr2, double ***pr5, double ***pr6)
{
   ali_nc (sc);
   ali_n_nl (lsf, 3);

   ald_nc_n (lrat, 3);
   ald_nc_n (clik, 3);
   ald_nc_nl (pr2);
   ald_nc_nl (pr5);
   ald_nc_nl (pr6);

   return;
}

void  free_y (int ****py)
{

   if (!*py)
      return;

   free_genes (&(*py)[0]);
   free_genes (&(*py)[1]);

   free (*py);
   *py = NULL;

   return;
}

void  free_ibd (int **g, int ***ix, int ***lab, int ***lst, int **np,
                int ***ct, double *****nsc, double **sss)
{
   int   c, s, i, j;

   if (!*g)
      return;

   for (i = 0; i <= MxGam; ++i)
   {
      if (*ix && (*ix)[i])
         free ((*ix)[i]);
      if (*lst && (*lst)[i])
         free ((*lst)[i]);
      if (*lab && (*lab)[i])
         free ((*lab)[i]);
   }
   free (*ix);
   *ix = NULL;
   free (*lst);
   *lst = NULL;
   free (*lab);
   *lab = NULL;
   free (*np);
   *np = NULL;
   free (*g);
   *g = NULL;

   if (*nsc)
   {
      for (c = 0; c < NScCmp; ++c)
         if ((*nsc)[c])
         {
            for (s = 0; s < NScSet[c]; ++s)
               if ((*nsc)[c][s])
               {
                  for (j = 0; j < NLoci; ++j)
                     if ((*nsc)[c][s][j])
                        free ((*nsc)[c][s][j]);
                  free ((*nsc)[c][s]);
               }
            free ((*nsc)[c]);
         }
      free (*nsc);
      *nsc = NULL;
   }
   fre_nc_n ((void *) ct);
   fre_n ((void *) sss);

   free_cst ();

   return;
}
void  free_lod (int **sc, int ***lsf, double ***lrat,
                double ***clik, double ***pr2,
                double ***pr5, double ***pr6)
{
   fre_n ((void *) sc);
   fre_n_n ((void *) lsf, 3);
   fre_nc_n ((void *) lrat);

   fre_nc_n ((void *) clik);
   fre_nc_n ((void *) pr2);
   fre_nc_n ((void *) pr5);
   fre_nc_n ((void *) pr6);

   return;
}

void  alloc_loglik (int ****rc, double ***pr1, double ***pr2,
                    double ***pr3)
/* int rc     [nc][nl][6]
 * double pr1 [nc][nl]
 * double pr2 [nc][nl]
 * double pr3 [nc][nl]
 */
{

   ali_nc_nl_n (rc, 6);
   ald_nc_nl (pr1);
   ald_nc_nl (pr2);
   ald_nc_nl (pr3);

   return;
}

void  free_loglik (int ****rc, double ***pr1, double ***pr2,
                   double ***pr3)
{
   fre_nc_nl_n ((void *) rc);
   fre_nc_n ((void *) pr1);
   fre_nc_n ((void *) pr2);
   fre_nc_n ((void *) pr3);
}

void  free_wndw (int **g, int **wl, double *****nsc, int nw, int nwpt)
{
   int   c, j, k;

   if (!*g)
      return;

   free (*g);
   *g = NULL;

   if (*wl)
      free (*wl);
   *wl = NULL;

   if (*nsc)
   {
      for (c = 0; c < NScCmpAlloc; ++c)
         if ((*nsc)[c])
         {
            for (j = 0; j < NScSet[c]; ++j)
               if ((*nsc)[c][j])
               {
                  for (k = 0; k < nw; ++k)
                     if ((*nsc)[c][j][k])
                        free ((*nsc)[c][j][k]);
                  free ((*nsc)[c][j]);
               }
            free ((*nsc)[c]);
         }
      free (*nsc);
      *nsc = NULL;
   }
   return;
}
