#include "quant.h"

#define GG(I,J)      gg[(cpsize+1) * (I+1) + (J+1)]

/*
 * Version of cheigen, to find eigenvalues of matrix. Uses reduce(), and
 * other tricks, to enable larger pedigrees to be handled. Original program,
 * 1992. 
 *
 * Modified, June 1996, by Chi-hong Tseng, to allow pedigree components
 * to be handled separately 
 *
 */

void  bigcheig (double *evec, int tnp, int cpsize, Ind ** cpnghd, int *cdex)
/*
 * evec will be the eigenvalues, tnp the dimension 
 *
 * ivec is the imaginary parts of evecs, and should be 0. zz is the
 * eigenvectors, which we don't want.  
 */

{
   int   i, mum, dad, j, jj;
   int   imm;
   int   jjj, ierr, n, nn;
   double z[1], *w, *fv1, *fv2, *dummy;
   int  *move_to;
   double *gg;

   if (ChkEigC)
      printf (" now in bigcheig(); observed G-dimension %d \n", tnp);

   /* compute G ; ASSUMES an ORDERED pedigree! */

   gg = malloc ((cpsize + 1) * (cpsize + 1) * sizeof (double));
   alloc_chk (WHERE, gg);

   gg[0] = 0.0;
   for (i = 0; i < cpsize; ++i)
      GG (i, -1) = 0.0;

   for (i = 0; i < cpsize; ++i)
   {
      if (cpnghd[i]->ma->index == INVALID_INDEX)
         mum = INVALID_INDEX;
      else
         mum = cdex[cpnghd[i]->ma->index];
      if (cpnghd[i]->pa->index == INVALID_INDEX)
         dad = INVALID_INDEX;
      else
         dad = cdex[cpnghd[i]->pa->index];

      GG (i, i) = 1.0 + 0.5 * GG (mum, dad);
      /*
       * NOTE: this relies on mum == INVALID_INDEX == -1 for founders i
       * GG(i, -1) == GG(0, -1) == 0.0 for all i If the definition of
       * INVALID_INDEX is changed it will break. 
       */
      for (j = 0; j < i; ++j)
      {
         GG (j, i) = 0.5 * (GG (j, mum) + GG (j, dad));
         GG (i, j) = GG (j, i);
      }
   }

   /*
    * reduce G by unobserved individuals; can still use gg-space the array
    * uses indices from 1 only, and unobserved rows/columns move_to 0, 
    *
    * combining this with reduction of G using Robin Thompson's proposal for a
    * symmetric reduced matrix  
    */

   move_to = malloc ((cpsize) * sizeof (int));
   alloc_chk (WHERE, move_to);

   imm = reduce (gg, evec, move_to, cdex, cpsize, cpnghd);

   if (ChkEigC)
      printf (" now reduce() called;  immediate evs %d,"
              " reduced dimension %d\n", imm, (tnp - imm));

   jj = -1;
   for (i = 0; i < cpsize; ++i)
   {
      if (move_to[i] == 0)
         move_to[i] = -1;
      else
      {
         ++jj;
         move_to[i] = jj;
         for (j = 0; j < i; ++j)
         {
            GG (move_to[i], move_to[j]) = GG (i, j);
            GG (move_to[j], move_to[i]) = GG (move_to[i], move_to[j]);
         }
         GG (move_to[i], move_to[i]) = GG (i, i);
      }
   }

   free ((void *) move_to);
   move_to = NULL;

   /* set up for fortran routine */
   jjj = 0;
   n = tnp - imm;
   nn = tnp - imm;

   dummy = malloc (tnp * tnp * sizeof (double));
   alloc_chk (WHERE, dummy);

   for (i = 1; i <= tnp - imm; ++i)
   {
      for (j = 1; j <= tnp - imm; ++j)
         dummy[i + (tnp - imm) * j - (tnp - imm) - 1] = GG (i - 1, j - 1);
   }
   free ((void *) gg);
   gg = NULL;

   w = malloc (cpsize * sizeof (double));
   alloc_chk (WHERE, w);
   fv1 = malloc (cpsize * sizeof (double));
   alloc_chk (WHERE, fv1);
   fv2 = malloc (cpsize * sizeof (double));
   alloc_chk (WHERE, fv2);

   /* call FORTRAN eigen routine */
   if (ChkEigC)
      printf (" about to call rs_()\n");
   rs_ (&nn, &n, dummy, w, &jjj, z, fv1, fv2, &ierr);

   if (ChkEigC)
      printf (" back out of rs_()\n");
   if (ierr)
      give_warn_where (WHERE, ERQ_rse, ierr);
   else if (ChkEigC)
      printf (" normal return from rs_() with ierr = %d\n\n", ierr);

   for (i = imm + 1; i <= tnp; ++i)
      evec[i] = w[i - imm - 1];

   free ((void *) dummy);
   dummy = NULL;
   free ((void *) w);
   w = NULL;
   free ((void *) fv1);
   fv1 = NULL;
   free ((void *) fv2);
   fv2 = NULL;

   return;
}
