#include "quant.h"

/*
 * To find terms of G-inverse for a large pedigree, using
 *     Henderson's algorithm, using only nghds terms. 
 *
 * Installed in Quant library, August, 1991. 
 *
 * Modified to permit inbred pedigrees,  Sept 1991. 
 * Note that use of the nghds structures  
 *     currently only permits founders and non-founders, no "half-founders";
 *     however, in the event this is ever changed, the terms required for
 *     halffounders are already incoporated (perhaps correctly). 
 */

static int row_set (int, int *, double *, double *);
static int countchild (int, Indlist *, int *, int *, double *, double *,
                       double *);

int   henderson (Term * gv)
{
   int  *index;
   double *value, *diag;
   int   n, k, i, j, foud, foum;
   Ind  *mum, *dad;

   index = malloc (pedsize * sizeof (int));
   alloc_chk (WHERE, index);
   value = malloc (pedsize * sizeof (double));
   alloc_chk (WHERE, value);
   diag = malloc (pedsize * sizeof (double));
   alloc_chk (WHERE, diag);

   /* for general (inbred) pedigree must first set diagonal terms */
   for (i = 0; i < pedsize; ++i)
   {
      mum = nghd[i].ma;
      dad = nghd[i].pa;
      foud = ((dad->index) == INVALID_INDEX);
      foum = ((mum->index) == INVALID_INDEX);
      diag[i] = 1.0;
      if (!foud && !foum)
      {
         diag[i] = inbreeding (dad) + inbreeding (mum);
         diag[i] = 1.0 / (0.5 - 0.25 * diag[i]);
      }
      if (!foud && foum)
      {
         diag[i] = inbreeding (dad);
         diag[i] = 1.0 / (0.75 - 0.25 * diag[i]);
      }
      if (foud && !foum)
      {
         diag[i] = inbreeding (mum);
         diag[i] = 1.0 / (0.75 - 0.25 * diag[i]);
      }
   }
   /* now compute matrix  */
   k = 0;
   for (i = 0; i < pedsize; ++i)
   {
      n = row_set (i, index, value, diag);
      for (j = 0; j < n; ++j)
      {
         ++k;
         gv[k].r1 = i;
         gv[k].r2 = index[j];
         gv[k].value = value[j];
      }
   }

   free ((void *) index);
   index = NULL;
   free ((void *) value);
   value = NULL;
   free ((void *) diag);
   diag = NULL;

   return (k);
}

static int row_set (int i, int *index, double *value, double *diag)
{
   int   count;
   int   foud, foum, dad, mum;
   double vv, vp, vtot;
   Marlist *marriage_ptr;
   int   nkids, spouse;

   count = 0;
   vv = diag[i];
   dad = nghd[i].pa->index;
   mum = nghd[i].ma->index;
   foud = (dad == INVALID_INDEX);
   foum = (mum == INVALID_INDEX);
   if (!foud && dad < i)
   {
      index[count] = dad;
      value[count] = -0.5 * vv;
      ++count;
   }
   if (!foum && mum < i)
   {
      index[count] = mum;
      value[count] = -0.5 * vv;
      ++count;
   }
   if ((!foum && foud) || (foud && !foum))
      give_warn_where (WHERE, ERQ_hff, i, "row_set");
   marriage_ptr = nghd[i].marriages;
   vtot = 0.0;
   while (marriage_ptr != NULL)
   {
      nkids = countchild (i, marriage_ptr->current->kids,
                          &count, index, value, diag, &vp);
      spouse = marriage_ptr->current->spouse->index;
      if (spouse < i)
      {
         index[count] = spouse;
         value[count] = 0.25 * vp * nkids;
         ++count;
      }
      vtot += 0.25 * vp * nkids;
      marriage_ptr = marriage_ptr->next;
   }
   index[count] = i;
   value[count] = vv + vtot;
   ++count;
   return (count);
}

static int countchild (int i, Indlist * head, int *cc, int *index,
                       double *value, double *diag, double *vk)
{
   int   nextkid;
   int   kids = 0;
   while (head != NULL)
   {
      nextkid = head->current->index;
      *vk = diag[nextkid];
      ++kids;
      if (nextkid < i)
      {
         index[*cc] = nextkid;
         value[*cc] = -0.5 * (*vk);
         ++(*cc);
      }
      head = (head->next);
   }
   return (kids);
}
