
  /*****************************************************************
   * L-Sample Step     SCH July  1999                              *
   *   to subroutine   EAT March 2000                              *
   *   modified for scan or random locus  EAT Dec 2000             *
   *                                                               *
   * L-peel likelihood compute;  EAT May 2001                      *
   *   rblik() returns log P(Y_T|S_M) for specified trait          *
   *   position; lsample_flag =0; S is NOT resampled               *
   *                                                               *
   *  do_rblik modified for batch-mean computation: 20-02-2002 EAT *
   *                                                               *
   *****************************************************************/
#include "lsample.h"
#include "msample.h"

double do_lsmpl (int nl, int jj, double **prob, int ***y, int ng, int qt)
{
   double z;
   int   err, c;
   set_transmission (loci, jj, nl + 1);
   if ((jj == 0) && (qt == 1))
      z = locus_mjrpeel (ym, loci[0].ve, 1.0, 1, loci);
   else
      z = locus_mjrpeel (0, 1.0, 1.0, 1, loci + jj);

   /* checks for Zero probability in peeling operation                        */
   if (z == -DBL_MAX)
      give_up_where (WHERE, ERS_zpp);

   (void) set_genes (jj);

   for (c = 0; c < n_compnts; ++c)
      prob[c][jj] = seg_pen2 (c, y, Qq[jj], jj, ng, &err);

   return z;
}

void  do_rblik (int ntp, int *lleft, int *lrght, double *rml, double *rfl,
                double *rmr, double *rfr, double *rb_prob, double *rb_bm,
                double base_lik, int qt)
/* ntp = # linked trait positions, at intervals loc and recombination rec,
   loc_left, etc, are recombinations, intervals etc, for trait positions.
   rb_prob is cumulative likelihood estimate
   transmissions MUST be computed by set_simple_trans ONLY
   lsample_flag  MUST be 0 -- S's are NOT resampled.
   base_lik = base (unlinked) likelihood, computed once only   */
{
   double z;
   int   k;

   for (k = 1; k < ntp; ++k)
   {
      if (lleft[k] != lrght[k])
      {
/*
 *  printf ("\n starting trait position %d", k); fflush(stdout);
 *  printf ("\n locus left %d recom %f %f,  right %d recom %f %f",
 *  lleft[k], rml[k], rfl[k], lrght[k], rmr[k], rfr[k]);
 *  fflush(stdout);
 */
         set_simple_trans (loci, lleft[k], lrght[k], rml[k],
                           rfl[k], rmr[k], rfr[k]);  /* new trans routine */

         if (qt == 1)
            z = locus_mjrpeel (ym, loci[0].ve, 1.0, 0, loci);
         else
            z = locus_mjrpeel (0, 1.0, 1.0, 0, loci);  /* trait locus peel;
                                                          sample_flag=0  */
         /*    printf ("\n completed peel; like = %e",z); fflush(stdout);  */

         /* checks for Zero probability in peeling operation                  */
         if (z == -DBL_MAX)
            give_up_where (WHERE, ERS_zpp);

         rb_prob[k] += exp (z - base_lik);
         rb_bm[k] += exp (z - base_lik);
/*
 *    printf ("\n end of loop for k=%d", k); fflush(stdout);
 *    printf ("   current rb_prob = %e", rb_prob[k]); fflush(stdout);
 */
      }
   }
   return;
}
